/// <reference types="node" />
import { EventEmitter } from "events";
import { Queue } from "./queue";
import { Connection } from "./connection";
import { SchedulerOptions } from "../types/options";
import { Job } from "../types/job";
import { Jobs } from "../types/jobs";
import { ErrorPayload } from "../types/errorPayload";
export declare interface Scheduler {
    options: SchedulerOptions;
    jobs: Jobs;
    name: string;
    master: boolean;
    running: boolean;
    processing: boolean;
    queue: Queue;
    connection: Connection;
    timer: NodeJS.Timeout;
    on(event: "start" | "end" | "poll" | "master", cb: () => void): this;
    on(event: "cleanStuckWorker", cb: (workerName: string, errorPayload: ErrorPayload, delta: number) => void): this;
    on(event: "error", cb: (error: Error, queue: string) => void): this;
    on(event: "workingTimestamp", cb: (timestamp: number) => void): this;
    on(event: "transferredJob", cb: (timestamp: number, job: Job<any>) => void): this;
    once(event: "start" | "end" | "poll" | "master", cb: () => void): this;
    once(event: "cleanStuckWorker", cb: (workerName: string, errorPayload: ErrorPayload, delta: number) => void): this;
    once(event: "error", cb: (error: Error, queue: string) => void): this;
    once(event: "workingTimestamp", cb: (timestamp: number) => void): this;
    once(event: "transferredJob", cb: (timestamp: number, job: Job<any>) => void): this;
    removeAllListeners(event: SchedulerEvent): this;
}
export declare type SchedulerEvent = "start" | "end" | "poll" | "master" | "cleanStuckWorker" | "error" | "workingTimestamp" | "transferredJob";
export declare class Scheduler extends EventEmitter {
    constructor(options: any, jobs?: {});
    connect(): Promise<void>;
    start(): Promise<void>;
    end(): Promise<unknown>;
    poll(): any;
    private pollAgainLater;
    private masterKey;
    private tryForMaster;
    private releaseMasterLock;
    private nextDelayedTimestamp;
    private enqueueDelayedItemsForTimestamp;
    private nextItemForTimestamp;
    private transfer;
    private cleanupTimestamp;
    private checkStuckWorkers;
    forceCleanWorker(workerName: any, delta: any): Promise<void>;
}
